@@ -3,6 +3,7 @@ package action
 import (
 	"context"
 	"fmt"
+	"io"
 	"time"
 
 	"github.com/k0sproject/k0sctl/phase"
@@ -12,6 +13,7 @@ import (
 type Backup struct {
 	// Manager is the phase manager
 	Manager *phase.Manager
+	Out     io.Writer
 }
 
 func (b Backup) Run(ctx context.Context) error {
@@ -19,16 +21,14 @@ func (b Backup) Run(ctx context.Context) error {
 
 	lockPhase := &phase.Lock{}
 
-	b.Manager.AddPhase(
-		&phase.Connect{},
-		&phase.DetectOS{},
-		lockPhase,
-		&phase.PrepareHosts{},
+    b.Manager.AddPhase(
+        &phase.Connect{},
+        &phase.DetectOS{},
+        lockPhase,
+        &phase.PrepareHosts{},
 		&phase.GatherFacts{SkipMachineIDs: true},
 		&phase.GatherK0sFacts{},
-		&phase.RunHooks{Stage: "before", Action: "backup"},
-		&phase.Backup{},
-		&phase.RunHooks{Stage: "after", Action: "backup"},
+		&phase.Backup{Out: b.Out},
 		&phase.Unlock{Cancel: lockPhase.Cancel},
 		&phase.Disconnect{},
 	)
