@@ -1,6 +1,7 @@
 package cmd
 
 import (
+	"bytes"
 	"context"
 	"fmt"
 	"io"
@@ -65,6 +66,15 @@ var (
 		Value: false,
 	}
 
+	forceFlag = &cli.BoolFlag{
+		Name:  "force",
+		Usage: "Attempt a forced operation in case of certain failures",
+		Action: func(c *cli.Context, force bool) error {
+			phase.Force = force
+			return nil
+		},
+	}
+
 	configFlag = &cli.StringSliceFlag{
 		Name:      "config",
 		Usage:     "Path or glob to config yaml. Can be given multiple times. Use '-' to read from stdin.",
@@ -187,11 +197,17 @@ func initConfig(ctx *cli.Context) error {
 	manifestReader := &manifest.Reader{}
 
 	for _, f := range configs {
-		file, err := configReader(f)
+		file, err := configReader(ctx, f)
 		if err != nil {
 			return err
 		}
-		defer file.Close()
+		cfgFile := file
+		cfgName := f
+		defer func() {
+			if err := cfgFile.Close(); err != nil {
+				log.Warnf("failed to close config file %s: %v", cfgName, err)
+			}
+		}()
 
 		content, err := io.ReadAll(file)
 		if err != nil {
@@ -202,12 +218,19 @@ func initConfig(ctx *cli.Context) error {
 		if err != nil {
 			return err
 		}
+		if bytes.Equal(subst, content) {
+			log.Debugf("no variable substitutions made in %s", f)
+		} else {
+			log.Debugf("variable substitutions made in %s, before %d after %d bytes", f, len(content), len(subst))
+		}
 
-		log.Debugf("Loaded configuration from %s:\n%s", f, subst)
+		log.Debugf("parsing configuration from %s", f)
 
 		if err := manifestReader.ParseBytes(subst); err != nil {
 			return fmt.Errorf("failed to parse config: %w", err)
 		}
+
+		log.Debugf("parsed %d resource definition manifests from %s", manifestReader.Len(), f)
 	}
 
 	if manifestReader.Len() == 0 {
@@ -220,9 +243,7 @@ func initConfig(ctx *cli.Context) error {
 }
 
 func displayCopyright(ctx *cli.Context) error {
-	fmt.Printf("k0sctl %s Copyright 2023, k0sctl authors.\n", k0sctl.Version)
-	fmt.Println("By continuing to use k0sctl you agree to these terms:")
-	fmt.Println("https://k0sproject.io/licenses/eula")
+	fmt.Fprintf(ctx.App.Writer, "k0sctl %s Copyright 2025, k0sctl authors.\n", k0sctl.Version)
 	return nil
 }
 
@@ -260,6 +281,9 @@ func readConfig(ctx *cli.Context) (*v1beta1.Cluster, error) {
 		return nil, fmt.Errorf("failed to unmarshal cluster config: %w", err)
 	}
 	if k0sConfigs, err := mr.GetResources("k0s.k0sproject.io/v1beta1", "ClusterConfig"); err == nil && len(k0sConfigs) > 0 {
+		if cfg.Spec.K0s.Config == nil {
+			cfg.Spec.K0s.Config = make(dig.Mapping)
+		}
 		for _, k0sConfig := range k0sConfigs {
 			k0s := make(dig.Mapping)
 			log.Debugf("unmarshalling %d bytes of config from %v", len(k0sConfig.Raw), k0sConfig.Filename())
@@ -305,9 +329,18 @@ func initManager(ctx *cli.Context) error {
 		return fmt.Errorf("failed to initialize phase manager: %w", err)
 	}
 
-	manager.Concurrency = ctx.Int("concurrency")
-	manager.ConcurrentUploads = ctx.Int("concurrent-uploads")
+	if ctx.IsSet("concurrency") {
+		manager.Concurrency = ctx.Int("concurrency")
+	} else {
+		manager.Concurrency = cfg.Spec.Options.Concurrency.Limit
+	}
+	if ctx.IsSet("concurrent-uploads") {
+		manager.ConcurrentUploads = ctx.Int("concurrent-uploads")
+	} else {
+		manager.ConcurrentUploads = cfg.Spec.Options.Concurrency.Uploads
+	}
 	manager.DryRun = ctx.Bool("dry-run")
+	manager.Writer = ctx.App.Writer
 
 	ctx.Context = context.WithValue(ctx.Context, ctxManagerKey{}, manager)
 
@@ -318,7 +351,7 @@ func initManager(ctx *cli.Context) error {
 func initLogging(ctx *cli.Context) error {
 	log.SetLevel(log.TraceLevel)
 	log.SetOutput(io.Discard)
-	initScreenLogger(logLevelFromCtx(ctx, log.InfoLevel))
+	initScreenLogger(ctx, logLevelFromCtx(ctx, log.InfoLevel))
 	exec.DisableRedact = ctx.Bool("no-redact")
 	rig.SetLogger(log.StandardLogger())
 	return initFileLogger(ctx)
@@ -330,7 +363,7 @@ func initSilentLogging(ctx *cli.Context) error {
 	log.SetLevel(log.TraceLevel)
 	log.SetOutput(io.Discard)
 	exec.DisableRedact = ctx.Bool("no-redact")
-	initScreenLogger(logLevelFromCtx(ctx, log.FatalLevel))
+	initScreenLogger(ctx, logLevelFromCtx(ctx, log.FatalLevel))
 	rig.SetLogger(log.StandardLogger())
 	return initFileLogger(ctx)
 }
@@ -345,8 +378,8 @@ func logLevelFromCtx(ctx *cli.Context, defaultLevel log.Level) log.Level {
 	}
 }
 
-func initScreenLogger(lvl log.Level) {
-	log.AddHook(screenLoggerHook(lvl))
+func initScreenLogger(ctx *cli.Context, lvl log.Level) {
+	log.AddHook(screenLoggerHook(ctx, lvl))
 }
 
 func initFileLogger(ctx *cli.Context) error {
@@ -372,24 +405,30 @@ func LogFile() (*os.File, error) {
 
 	logFile, err := os.OpenFile(fn, os.O_RDWR|os.O_CREATE|os.O_APPEND|os.O_SYNC, 0o600)
 	if err != nil {
-		return nil, fmt.Errorf("Failed to open log %s: %s", fn, err.Error())
+		return nil, fmt.Errorf("failed to open log %s: %s", fn, err.Error())
 	}
 
-	_, _ = fmt.Fprintf(logFile, "time=\"%s\" level=info msg=\"###### New session ######\"\n", time.Now().Format(time.RFC822))
+	fmt.Fprintf(logFile, "time=\"%s\" level=info msg=\"###### New session ######\"\n", time.Now().Format(time.RFC822))
 
 	return logFile, nil
 }
 
-func configReader(f string) (io.ReadCloser, error) {
+func configReader(ctx *cli.Context, f string) (io.ReadCloser, error) {
 	if f == "-" {
-		stat, err := os.Stdin.Stat()
-		if err != nil {
-			return nil, fmt.Errorf("can't stat stdin: %s", err.Error())
+		if inF, ok := ctx.App.Reader.(*os.File); ok {
+			stat, err := inF.Stat()
+			if err != nil {
+				return nil, fmt.Errorf("can't stat stdin: %s", err.Error())
+			}
+			if (stat.Mode() & os.ModeCharDevice) == 0 {
+				return inF, nil
+			}
+			return nil, fmt.Errorf("can't read stdin")
 		}
-		if (stat.Mode() & os.ModeCharDevice) == 0 {
-			return os.Stdin, nil
+		if inCloser, ok := ctx.App.Reader.(io.ReadCloser); ok {
+			return inCloser, nil
 		}
-		return nil, fmt.Errorf("can't read stdin")
+		return io.NopCloser(ctx.App.Reader), nil
 	}
 
 	variants := []string{f}
@@ -448,16 +487,17 @@ func (h *loghook) Fire(entry *log.Entry) error {
 	return err
 }
 
-func screenLoggerHook(lvl log.Level) *loghook {
+func screenLoggerHook(ctx *cli.Context, lvl log.Level) *loghook {
 	var forceColors bool
-	var writer io.Writer
+	writer := ctx.App.Writer
 	if runtime.GOOS == "windows" {
-		writer = ansicolor.NewAnsiColorWriter(os.Stdout)
+		writer = ansicolor.NewAnsiColorWriter((ctx.App.Writer))
 		forceColors = true
 	} else {
-		writer = os.Stdout
-		if fi, _ := os.Stdout.Stat(); (fi.Mode() & os.ModeCharDevice) != 0 {
-			forceColors = true
+		if outF, ok := writer.(*os.File); ok {
+			if fi, _ := outF.Stat(); (fi.Mode() & os.ModeCharDevice) != 0 {
+				forceColors = true
+			}
 		}
 	}
 
@@ -491,8 +531,8 @@ func fileLoggerHook(logFile io.Writer) *loghook {
 	return l
 }
 
-func displayLogo(_ *cli.Context) error {
-	fmt.Print(logo)
+func displayLogo(ctx *cli.Context) error {
+	fmt.Fprint(ctx.App.Writer, logo)
 	return nil
 }
 
