@@ -1,6 +1,8 @@
 package phase
 
 import (
+	"context"
+
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
 	log "github.com/sirupsen/logrus"
 )
@@ -21,8 +23,8 @@ func (p *DaemonReload) ShouldRun() bool {
 }
 
 // Run the phase
-func (p *DaemonReload) Run() error {
-	return p.parallelDo(p.Config.Spec.Hosts, func(h *cluster.Host) error {
+func (p *DaemonReload) Run(ctx context.Context) error {
+	return p.parallelDo(ctx, p.Config.Spec.Hosts, func(_ context.Context, h *cluster.Host) error {
 		log.Infof("%s: reloading service manager", h)
 		if err := h.Configurer.DaemonReload(h); err != nil {
 			log.Warnf("%s: failed to reload service manager: %s", h, err.Error())
