@@ -61,7 +61,7 @@ func (p *DownloadK0s) Run(ctx context.Context) error {
 }
 
 func (p *DownloadK0s) downloadK0s(_ context.Context, h *cluster.Host) error {
-	tmp := h.Configurer.K0sBinaryPath() + ".tmp." + strconv.Itoa(int(time.Now().UnixNano()))
+	tmp := h.K0sInstallLocation() + ".tmp." + strconv.Itoa(int(time.Now().UnixNano()))
 
 	log.Infof("%s: downloading k0s %s", h, p.Config.Spec.K0s.Version)
 	if h.K0sDownloadURL != "" {
@@ -82,3 +82,13 @@ func (p *DownloadK0s) downloadK0s(_ context.Context, h *cluster.Host) error {
 
 	return nil
 }
+
+// Cleanup removes the binary temp file if it wasn't used
+func (p *DownloadK0s) CleanUp() {
+	_ = p.parallelDo(context.Background(), p.hosts, func(_ context.Context, h *cluster.Host) error {
+		if h.Metadata.K0sBinaryTempFile != "" {
+			_ = h.Configurer.DeleteFile(h, h.Metadata.K0sBinaryTempFile)
+		}
+		return nil
+	})
+}
