@@ -50,7 +50,7 @@ func (p *InstallBinaries) DryRun() error {
 		context.Background(),
 		p.Config.Spec.Hosts.Filter(func(h *cluster.Host) bool { return h.Metadata.K0sBinaryTempFile != "" }),
 		func(_ context.Context, h *cluster.Host) error {
-			p.DryMsgf(h, "install k0s %s binary from %s to %s", p.Config.Spec.K0s.Version, h.Metadata.K0sBinaryTempFile, h.Configurer.K0sBinaryPath())
+			p.DryMsgf(h, "install k0s %s binary from %s to %s", p.Config.Spec.K0s.Version, h.Metadata.K0sBinaryTempFile, h.K0sInstallLocation())
 			if err := h.Execf(`chmod +x "%s"`, h.Metadata.K0sBinaryTempFile, exec.Sudo(h)); err != nil {
 				logrus.Warnf("%s: failed to chmod k0s temp binary for dry-run: %s", h, err.Error())
 			}
@@ -70,6 +70,7 @@ func (p *InstallBinaries) installBinary(_ context.Context, h *cluster.Host) erro
 	if err := h.UpdateK0sBinary(h.Metadata.K0sBinaryTempFile, p.Config.Spec.K0s.Version); err != nil {
 		return fmt.Errorf("failed to install k0s binary: %w", err)
 	}
+	h.Metadata.K0sBinaryTempFile = ""
 
 	return nil
 }
@@ -80,9 +81,7 @@ func (p *InstallBinaries) CleanUp() {
 			return nil
 		}
 		logrus.Infof("%s: cleaning up k0s binary tempfile", h)
-		if err := h.Configurer.DeleteFile(h, h.Metadata.K0sBinaryTempFile); err != nil {
-			return fmt.Errorf("clean up tempfile: %w", err)
-		}
+		_ = h.Configurer.DeleteFile(h, h.Metadata.K0sBinaryTempFile)
 		return nil
 	})
 	if err != nil {
