@@ -83,7 +83,7 @@ func (p *Reinstall) reinstall(ctx context.Context, h *cluster.Host) error {
 		return err
 	}
 	log.Infof("%s: reinstalling k0s", h)
-	err = p.Wet(h, fmt.Sprintf("reinstall k0s using `%s", strings.ReplaceAll(cmd, h.Configurer.K0sBinaryPath(), "k0s")), func() error {
+	err = p.Wet(h, fmt.Sprintf("reinstall k0s using `%s", strings.ReplaceAll(cmd, h.K0sInstallLocation(), "k0s")), func() error {
 		if err := h.Exec(cmd, exec.Sudo(h)); err != nil {
 			return fmt.Errorf("failed to reinstall k0s: %w", err)
 		}
@@ -98,7 +98,7 @@ func (p *Reinstall) reinstall(ctx context.Context, h *cluster.Host) error {
 			return fmt.Errorf("failed to restart k0s: %w", err)
 		}
 		log.Infof("%s: waiting for the k0s service to start", h)
-		if err := retry.AdaptiveTimeout(ctx, retry.DefaultTimeout, node.ServiceRunningFunc(h, h.K0sServiceName())); err != nil {
+		if err := retry.WithDefaultTimeout(ctx, node.ServiceRunningFunc(h, h.K0sServiceName())); err != nil {
 			return fmt.Errorf("k0s did not restart: %w", err)
 		}
 		return nil
