@@ -31,12 +31,22 @@ func (p *ResetControllers) Title() string {
 	return "Reset controllers"
 }
 
+// Before runs "before reset" hooks
+func (p *ResetControllers) Before() error {
+	return p.runHooks(context.Background(), "reset", "before", p.hosts...)
+}
+
+// After runs "after reset" hooks
+func (p *ResetControllers) After() error {
+	return p.runHooks(context.Background(), "reset", "after", p.hosts...)
+}
+
 // Prepare the phase
 func (p *ResetControllers) Prepare(config *v1beta1.Cluster) error {
 	p.Config = config
 	p.leader = p.Config.Spec.K0sLeader()
 
-	var controllers cluster.Hosts = p.Config.Spec.Hosts.Controllers()
+	controllers := p.Config.Spec.Hosts.Controllers()
 	log.Debugf("%d controllers in total", len(controllers))
 	p.hosts = controllers.Filter(func(h *cluster.Host) bool {
 		return h.Reset
@@ -50,23 +60,40 @@ func (p *ResetControllers) ShouldRun() bool {
 	return len(p.hosts) > 0
 }
 
+// DryRun reports nodes that would get reset
+func (p *ResetControllers) DryRun() error {
+	for _, h := range p.hosts {
+		p.DryMsg(h, "reset node")
+	}
+	return nil
+}
+
 // Run the phase
 func (p *ResetControllers) Run(ctx context.Context) error {
 	for _, h := range p.hosts {
-		log.Debugf("%s: draining node", h)
+		if t := p.Config.Spec.Options.EvictTaint; t.Enabled && t.ControllerWorkers && h.Role != "controller" {
+			log.Debugf("%s: add taint: %s", h, t.String())
+			if err := p.leader.AddTaint(h, t.String()); err != nil {
+				return fmt.Errorf("add taint: %w", err)
+			}
+		}
 		if !p.NoDrain && h.Role != "controller" {
-			if err := p.leader.DrainNode(&cluster.Host{
-				Metadata: cluster.HostMetadata{
-					Hostname: h.Metadata.Hostname,
+			log.Debugf("%s: draining node", h)
+			if err := p.leader.DrainNode(
+				&cluster.Host{
+					Metadata: cluster.HostMetadata{
+						Hostname: h.Metadata.Hostname,
+					},
 				},
-			}); err != nil {
+				p.Config.Spec.Options.Drain,
+			); err != nil {
 				log.Warnf("%s: failed to drain node: %s", h, err.Error())
 			}
 		}
 		log.Debugf("%s: draining node completed", h)
 
-		log.Debugf("%s: deleting node...", h)
 		if !p.NoDelete && h.Role != "controller" {
+			log.Debugf("%s: deleting node...", h)
 			if err := p.leader.DeleteNode(&cluster.Host{
 				Metadata: cluster.HostMetadata{
 					Hostname: h.Metadata.Hostname,
@@ -75,7 +102,6 @@ func (p *ResetControllers) Run(ctx context.Context) error {
 				log.Warnf("%s: failed to delete node: %s", h, err.Error())
 			}
 		}
-		log.Debugf("%s: deleting node", h)
 
 		if h.Configurer.ServiceIsRunning(h, h.K0sServiceName()) {
 			log.Debugf("%s: stopping k0s...", h)
@@ -83,7 +109,7 @@ func (p *ResetControllers) Run(ctx context.Context) error {
 				log.Warnf("%s: failed to stop k0s: %s", h, err.Error())
 			}
 			log.Debugf("%s: waiting for k0s to stop", h)
-			if err := retry.AdaptiveTimeout(ctx, retry.DefaultTimeout, node.ServiceStoppedFunc(h, h.K0sServiceName())); err != nil {
+			if err := retry.WithDefaultTimeout(ctx, node.ServiceStoppedFunc(h, h.K0sServiceName())); err != nil {
 				log.Warnf("%s: failed to wait for k0s to stop: %v", h, err)
 			}
 			log.Debugf("%s: stopping k0s completed", h)
@@ -115,6 +141,12 @@ func (p *ResetControllers) Run(ctx context.Context) error {
 		}
 		log.Debugf("%s: removing config completed", h)
 
+		log.Debugf("%s: removing k0s binary...", h)
+		if dErr := h.Configurer.DeleteFile(h, h.Configurer.K0sBinaryPath()); dErr != nil {
+			log.Warnf("%s: failed to remove existing binary %s: %s", h, h.Configurer.K0sConfigPath(), dErr)
+		}
+		log.Debugf("%s: removing binary completed", h)
+
 		if len(h.Environment) > 0 {
 			if err := h.Configurer.CleanupServiceEnvironment(h, h.K0sServiceName()); err != nil {
 				log.Warnf("%s: failed to clean up service environment: %s", h, err.Error())
