@@ -35,7 +35,7 @@ func (p *ResetWorkers) Prepare(config *v1beta1.Cluster) error {
 	p.Config = config
 	p.leader = p.Config.Spec.K0sLeader()
 
-	var workers cluster.Hosts = p.Config.Spec.Hosts.Workers()
+	workers := p.Config.Spec.Hosts.Workers()
 	log.Debugf("%d workers in total", len(workers))
 	p.hosts = workers.Filter(func(h *cluster.Host) bool {
 		return h.Reset
@@ -44,21 +44,48 @@ func (p *ResetWorkers) Prepare(config *v1beta1.Cluster) error {
 	return nil
 }
 
+// Before runs "before reset" hooks
+func (p *ResetWorkers) Before() error {
+	return p.runHooks(context.Background(), "reset", "before", p.hosts...)
+}
+
+// After runs "after reset" hooks
+func (p *ResetWorkers) After() error {
+	return p.runHooks(context.Background(), "reset", "after", p.hosts...)
+}
+
 // ShouldRun is true when there are workers that needs to be reset
 func (p *ResetWorkers) ShouldRun() bool {
 	return len(p.hosts) > 0
 }
 
+// DryRun reports the nodes will be reset
+func (p *ResetWorkers) DryRun() error {
+	for _, h := range p.hosts {
+		p.DryMsg(h, "node would be reset")
+	}
+	return nil
+}
+
 // Run the phase
 func (p *ResetWorkers) Run(ctx context.Context) error {
 	return p.parallelDo(ctx, p.hosts, func(_ context.Context, h *cluster.Host) error {
-		log.Debugf("%s: draining node", h)
+		if t := p.Config.Spec.Options.EvictTaint; t.Enabled {
+			log.Debugf("%s: add taint: %s", h, t.String())
+			if err := p.leader.AddTaint(h, t.String()); err != nil {
+				return fmt.Errorf("add taint: %w", err)
+			}
+		}
 		if !p.NoDrain {
-			if err := p.leader.DrainNode(&cluster.Host{
-				Metadata: cluster.HostMetadata{
-					Hostname: h.Metadata.Hostname,
+			log.Debugf("%s: draining node", h)
+			if err := p.leader.DrainNode(
+				&cluster.Host{
+					Metadata: cluster.HostMetadata{
+						Hostname: h.Metadata.Hostname,
+					},
 				},
-			}); err != nil {
+				p.Config.Spec.Options.Drain,
+			); err != nil {
 				log.Warnf("%s: failed to drain node: %s", h, err.Error())
 			}
 		}
@@ -82,7 +109,7 @@ func (p *ResetWorkers) Run(ctx context.Context) error {
 				log.Warnf("%s: failed to stop k0s: %s", h, err.Error())
 			}
 			log.Debugf("%s: waiting for k0s to stop", h)
-			if err := retry.AdaptiveTimeout(ctx, retry.DefaultTimeout, node.ServiceStoppedFunc(h, h.K0sServiceName())); err != nil {
+			if err := retry.WithDefaultTimeout(ctx, node.ServiceStoppedFunc(h, h.K0sServiceName())); err != nil {
 				log.Warnf("%s: failed to wait for k0s to stop: %s", h, err.Error())
 			}
 			log.Debugf("%s: stopping k0s completed", h)
@@ -90,7 +117,7 @@ func (p *ResetWorkers) Run(ctx context.Context) error {
 
 		log.Debugf("%s: resetting k0s...", h)
 		var stdoutbuf, stderrbuf bytes.Buffer
-		cmd, err := h.ExecStreams(h.Configurer.K0sCmdf("reset --data-dir=%s", h.K0sDataDir()), nil, &stdoutbuf, &stderrbuf, exec.Sudo(h))
+		cmd, err := h.ExecStreams(h.K0sResetCommand(), nil, &stdoutbuf, &stderrbuf, exec.Sudo(h))
 		if err != nil {
 			return fmt.Errorf("failed to run k0s reset: %w", err)
 		}
@@ -105,6 +132,12 @@ func (p *ResetWorkers) Run(ctx context.Context) error {
 		}
 		log.Debugf("%s: removing config completed", h)
 
+		log.Debugf("%s: removing k0s binary...", h)
+		if dErr := h.Configurer.DeleteFile(h, h.Configurer.K0sBinaryPath()); dErr != nil {
+			log.Warnf("%s: failed to remove existing binary %s: %s", h, h.Configurer.K0sConfigPath(), dErr)
+		}
+		log.Debugf("%s: removing binary completed", h)
+
 		if len(h.Environment) > 0 {
 			if err := h.Configurer.CleanupServiceEnvironment(h, h.K0sServiceName()); err != nil {
 				log.Warnf("%s: failed to clean up service environment: %s", h, err.Error())
