@@ -3,7 +3,7 @@ package phase
 import (
 	"context"
 	"fmt"
-	"time"
+	"slices"
 
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
@@ -29,7 +29,7 @@ func (p *UpgradeControllers) Title() string {
 func (p *UpgradeControllers) Prepare(config *v1beta1.Cluster) error {
 	log.Debugf("UpgradeControllers phase prep starting")
 	p.Config = config
-	var controllers cluster.Hosts = p.Config.Spec.Hosts.Controllers()
+	controllers := p.Config.Spec.Hosts.Controllers()
 	log.Debugf("%d controllers in total", len(controllers))
 	p.hosts = controllers.Filter(func(h *cluster.Host) bool {
 		if h.Metadata.K0sBinaryTempFile == "" {
@@ -46,6 +46,22 @@ func (p *UpgradeControllers) ShouldRun() bool {
 	return len(p.hosts) > 0
 }
 
+// Before runs "before upgrade" hooks for controller hosts that need upgrade
+func (p *UpgradeControllers) Before() error {
+	if len(p.hosts) == 0 {
+		return nil
+	}
+	return p.runHooks(context.Background(), "upgrade", "before", p.hosts...)
+}
+
+// After runs "after upgrade" hooks for controller hosts that were upgraded
+func (p *UpgradeControllers) After() error {
+	if len(p.hosts) == 0 {
+		return nil
+	}
+	return p.runHooks(context.Background(), "upgrade", "after", p.hosts...)
+}
+
 // CleanUp cleans up the environment override files on hosts
 func (p *UpgradeControllers) CleanUp() {
 	for _, h := range p.hosts {
@@ -63,13 +79,40 @@ func (p *UpgradeControllers) Run(ctx context.Context) error {
 		if !h.Configurer.FileExist(h, h.Metadata.K0sBinaryTempFile) {
 			return fmt.Errorf("k0s binary tempfile not found on host")
 		}
+
 		log.Infof("%s: starting upgrade", h)
+
+		if t := p.Config.Spec.Options.EvictTaint; t.Enabled && t.ControllerWorkers && h.Role != "controller" {
+			leader := p.Config.Spec.K0sLeader()
+			err := p.Wet(leader, "apply taint to node", func() error {
+				log.Warnf("%s: add taint %s on %s", leader, t.String(), h)
+				if err := leader.AddTaint(h, t.String()); err != nil {
+					return fmt.Errorf("add taint: %w", err)
+				}
+				log.Debugf("%s: wait for taint to be applied", h)
+				err := retry.WithDefaultTimeout(ctx, func(_ context.Context) error {
+					taints, err := leader.Taints(h)
+					if err != nil {
+						return fmt.Errorf("get taints: %w", err)
+					}
+					if !slices.Contains(taints, t.String()) {
+						return fmt.Errorf("taint %s not found", t.String())
+					}
+					return nil
+				})
+				return err
+			})
+			if err != nil {
+				return fmt.Errorf("apply taint: %w", err)
+			}
+		}
+
 		log.Debugf("%s: stop service", h)
 		err := p.Wet(h, "stop k0s service", func() error {
 			if err := h.Configurer.StopService(h, h.K0sServiceName()); err != nil {
 				return err
 			}
-			if err := retry.AdaptiveTimeout(ctx, retry.DefaultTimeout, node.ServiceStoppedFunc(h, h.K0sServiceName())); err != nil {
+			if err := retry.WithDefaultTimeout(ctx, node.ServiceStoppedFunc(h, h.K0sServiceName())); err != nil {
 				return fmt.Errorf("wait for k0s service stop: %w", err)
 			}
 			return nil
@@ -123,7 +166,7 @@ func (p *UpgradeControllers) Run(ctx context.Context) error {
 				return err
 			}
 			log.Infof("%s: waiting for the k0s service to start", h)
-			if err := retry.AdaptiveTimeout(ctx, retry.DefaultTimeout, node.ServiceRunningFunc(h, h.K0sServiceName())); err != nil {
+			if err := retry.WithDefaultTimeout(ctx, node.ServiceRunningFunc(h, h.K0sServiceName())); err != nil {
 				return fmt.Errorf("k0s service start: %w", err)
 			}
 			return nil
@@ -133,7 +176,7 @@ func (p *UpgradeControllers) Run(ctx context.Context) error {
 		}
 
 		if p.IsWet() {
-			err := retry.AdaptiveTimeout(ctx, 30*time.Second, func(_ context.Context) error {
+			err := retry.WithDefaultTimeout(ctx, func(_ context.Context) error {
 				out, err := h.ExecOutput(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "get --raw='/readyz?verbose=true'"), exec.Sudo(h))
 				if err != nil {
 					return fmt.Errorf("readiness endpoint reports %q: %w", out, err)
@@ -145,13 +188,21 @@ func (p *UpgradeControllers) Run(ctx context.Context) error {
 			}
 		}
 
-		h.Metadata.K0sRunningVersion = p.Config.Spec.K0s.Version
-	}
+		if t := p.Config.Spec.Options.EvictTaint; t.Enabled && t.ControllerWorkers && h.Role != "controller" {
+			leader := p.Config.Spec.K0sLeader()
+			err := p.Wet(leader, "remove taint from node", func() error {
+				log.Infof("%s: remove taint %s on %s", leader, t.String(), h)
+				if err := leader.RemoveTaint(h, t.String()); err != nil {
+					return fmt.Errorf("remove taint: %w", err)
+				}
+				return nil
+			})
+			if err != nil {
+				log.Warnf("%s: failed to remove taint %s on %s: %s", leader, t.String(), h, err.Error())
+			}
+		}
 
-	leader := p.Config.Spec.K0sLeader()
-	if NoWait || !p.IsWet() {
-		log.Warnf("%s: skipping scheduler and system pod checks because --no-wait given", leader)
-		return nil
+		h.Metadata.K0sRunningVersion = p.Config.Spec.K0s.Version
 	}
 
 	return nil
