@@ -44,7 +44,7 @@ func (p *UploadK0s) Prepare(config *v1beta1.Cluster) error {
 		}
 
 		// If the file has been changed compared to local, re-upload and replace
-		return h.FileChanged(h.UploadBinaryPath, h.Configurer.K0sBinaryPath())
+		return h.FileChanged(h.UploadBinaryPath, h.K0sInstallLocation())
 	})
 	return nil
 }
@@ -60,7 +60,7 @@ func (p *UploadK0s) Run(ctx context.Context) error {
 }
 
 func (p *UploadK0s) uploadBinary(_ context.Context, h *cluster.Host) error {
-	tmp := h.Configurer.K0sBinaryPath() + ".tmp." + strconv.Itoa(int(time.Now().UnixNano()))
+	tmp := h.K0sInstallLocation() + ".tmp." + strconv.Itoa(int(time.Now().UnixNano()))
 
 	stat, err := os.Stat(h.UploadBinaryPath)
 	if err != nil {
@@ -68,7 +68,7 @@ func (p *UploadK0s) uploadBinary(_ context.Context, h *cluster.Host) error {
 	}
 
 	log.Infof("%s: uploading k0s binary from %s to %s", h, h.UploadBinaryPath, tmp)
-	if err := h.Upload(h.UploadBinaryPath, tmp, exec.Sudo(h), exec.LogError(true)); err != nil {
+	if err := h.Upload(h.UploadBinaryPath, tmp, 0o600, exec.Sudo(h), exec.LogError(true)); err != nil {
 		return fmt.Errorf("upload k0s binary: %w", err)
 	}
 
@@ -83,3 +83,13 @@ func (p *UploadK0s) uploadBinary(_ context.Context, h *cluster.Host) error {
 
 	return nil
 }
+
+// Cleanup removes the binary temp file if it wasn't used
+func (p *UploadK0s) CleanUp() {
+	_ = p.parallelDo(context.Background(), p.hosts, func(_ context.Context, h *cluster.Host) error {
+		if h.Metadata.K0sBinaryTempFile != "" {
+			_ = h.Configurer.DeleteFile(h, h.Metadata.K0sBinaryTempFile)
+		}
+		return nil
+	})
+}
