@@ -1,10 +1,12 @@
 package v1beta1
 
 import (
+	"bytes"
 	"fmt"
 
 	"github.com/creasty/defaults"
 	"github.com/jellydator/validation"
+	"gopkg.in/yaml.v2"
 
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
 )
@@ -31,9 +33,7 @@ type Cluster struct {
 
 // UnmarshalYAML sets in some sane defaults when unmarshaling the data from yaml
 func (c *Cluster) UnmarshalYAML(unmarshal func(interface{}) error) error {
-	c.Metadata = &ClusterMetadata{
-		Name: "k0s-cluster",
-	}
+	c.Metadata = &ClusterMetadata{}
 	c.Spec = &cluster.Spec{}
 
 	type clusterConfig Cluster
@@ -50,6 +50,34 @@ func (c *Cluster) UnmarshalYAML(unmarshal func(interface{}) error) error {
 	return nil
 }
 
+// String renders the config as a string
+func (c *Cluster) String() string {
+	var buf bytes.Buffer
+	enc := yaml.NewEncoder(&buf)
+	if err := enc.Encode(c); err != nil {
+		return "# error enconding cluster config: " + err.Error()
+	}
+	return buf.String()
+}
+
+// SetDefaults initializes default values
+func (c *Cluster) SetDefaults() {
+	if c.Metadata == nil {
+		c.Metadata = &ClusterMetadata{}
+	}
+	if c.Spec == nil {
+		c.Spec = &cluster.Spec{}
+	}
+	_ = defaults.Set(c.Metadata)
+	_ = defaults.Set(c.Spec)
+	if defaults.CanUpdate(c.APIVersion) {
+		c.APIVersion = APIVersion
+	}
+	if defaults.CanUpdate(c.Kind) {
+		c.Kind = "Cluster"
+	}
+}
+
 // Validate performs a configuration sanity check
 func (c *Cluster) Validate() error {
 	validation.ErrorTag = "yaml"
