@@ -1,10 +1,12 @@
 package cluster
 
 import (
-	"fmt"
-	"net/url"
-	gos "os"
-	gopath "path"
+    "context"
+    "fmt"
+    "net/url"
+    gos "os"
+    gopath "path"
+    "slices"
 	"strings"
 	"time"
 
@@ -32,9 +34,11 @@ type Host struct {
 	PrivateInterface string            `yaml:"privateInterface,omitempty"`
 	PrivateAddress   string            `yaml:"privateAddress,omitempty"`
 	DataDir          string            `yaml:"dataDir,omitempty"`
+	KubeletRootDir   string            `yaml:"kubeletRootDir,omitempty"`
 	Environment      map[string]string `yaml:"environment,flow,omitempty"`
 	UploadBinary     bool              `yaml:"uploadBinary,omitempty"`
 	K0sBinaryPath    string            `yaml:"k0sBinaryPath,omitempty"`
+	K0sInstallPath   string            `yaml:"k0sInstallPath,omitempty"`
 	K0sDownloadURL   string            `yaml:"k0sDownloadURL,omitempty"`
 	InstallFlags     Flags             `yaml:"installFlags,omitempty"`
 	Files            []*UploadFile     `yaml:"files,omitempty"`
@@ -75,6 +79,14 @@ func (h *Host) SetDefaults() {
 		h.InstallFlags.Delete("--data-dir")
 		h.DataDir = dd
 	}
+
+	if krd := h.InstallFlags.GetValue("--kubelet-root-dir"); krd != "" {
+		if h.KubeletRootDir != "" {
+			log.Debugf("%s: changed kubeletRootDir from '%s' to '%s' because of --kubelet-root-dir installFlag", h, h.DataDir, krd)
+		}
+		h.InstallFlags.Delete("--kubelet-root-dir")
+		h.KubeletRootDir = krd
+	}
 }
 
 func validateBalancedQuotes(val any) error {
@@ -169,6 +181,7 @@ type configurer interface {
 	UpsertFile(os.Host, string, string) error
 	MachineID(os.Host) (string, error)
 	SetPath(string, string)
+	SystemTime(os.Host) (time.Time, error)
 }
 
 // HostMetadata resolved metadata for host
@@ -249,6 +262,15 @@ func (h *Host) ResolveConfigurer() error {
 	return fmt.Errorf("unsupported OS")
 }
 
+// K0sInstallLocation returns the k0s binary path from the K0sInstallPath field or configurer.K0sBinaryPath()
+func (h *Host) K0sInstallLocation() string {
+	if h.K0sInstallPath != "" {
+		return h.K0sInstallPath
+	}
+
+	return h.Configurer.K0sBinaryPath()
+}
+
 // K0sJoinTokenPath returns the token file path from install flags or configurer
 func (h *Host) K0sJoinTokenPath() string {
 	if path := h.InstallFlags.GetValue("--token-file"); path != "" {
@@ -285,6 +307,10 @@ func (h *Host) K0sInstallFlags() (Flags, error) {
 
 	flags.AddOrReplace(fmt.Sprintf("--data-dir=%s", shellescape.Quote(h.K0sDataDir())))
 
+	if h.KubeletRootDir != "" {
+		flags.AddOrReplace(fmt.Sprintf("--kubelet-root-dir=%s", shellescape.Quote(h.KubeletRootDir)))
+	}
+
 	switch h.Role {
 	case "controller+worker":
 		flags.AddUnlessExist("--enable-worker=true")
@@ -347,6 +373,17 @@ func (h *Host) K0sInstallCommand() (string, error) {
 	return h.Configurer.K0sCmdf("install %s %s", h.K0sRole(), flags.Join()), nil
 }
 
+// K0sResetCommand returns a full command that will reset k0s
+func (h *Host) K0sResetCommand() string {
+	var flags Flags
+	flags.Add(fmt.Sprintf("--data-dir=%s", shellescape.Quote(h.K0sDataDir())))
+	if h.KubeletRootDir != "" {
+		flags.Add(fmt.Sprintf("--kubelet-root-dir=%s", shellescape.Quote(h.KubeletRootDir)))
+	}
+
+	return h.Configurer.K0sCmdf("reset %s", flags.Join())
+}
+
 // K0sBackupCommand returns a full command to be used as run k0s backup
 func (h *Host) K0sBackupCommand(targetDir string) string {
 	return h.Configurer.K0sCmdf("backup --save-path %s --data-dir %s", shellescape.Quote(targetDir), h.K0sDataDir())
@@ -373,7 +410,7 @@ func (h *Host) K0sServiceName() string {
 }
 
 func (h *Host) k0sBinaryPathDir() string {
-	return gopath.Dir(h.Configurer.K0sBinaryPath())
+	return gopath.Dir(h.K0sInstallLocation())
 }
 
 // InstallK0sBinary installs the k0s binary from the provided file path to K0sBinaryPath
@@ -387,10 +424,14 @@ func (h *Host) InstallK0sBinary(path string) error {
 		return fmt.Errorf("create k0s binary dir: %w", err)
 	}
 
-	if err := h.Execf(`install -m 0750 -o root -g root "%s" "%s"`, path, h.Configurer.K0sBinaryPath(), exec.Sudo(h)); err != nil {
+	if err := h.Execf(`install -m 0750 -o root -g root "%s" "%s"`, path, h.K0sInstallLocation(), exec.Sudo(h)); err != nil {
 		return fmt.Errorf("install k0s binary: %w", err)
 	}
 
+	if err := h.Configurer.DeleteFile(h, path); err != nil {
+		log.Warnf("%s: failed to delete k0s binary tempfile: %s", h, err)
+	}
+
 	return nil
 }
 
@@ -423,8 +464,8 @@ func (h *Host) K0sDataDir() string {
 }
 
 // DrainNode drains the given node
-func (h *Host) DrainNode(node *Host) error {
-	return h.Exec(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "drain --grace-period=120 --force --timeout=5m --ignore-daemonsets --delete-emptydir-data %s", node.Metadata.Hostname), exec.Sudo(h))
+func (h *Host) DrainNode(node *Host, options DrainOption) error {
+	return h.Exec(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "drain %s %s", options.ToKubectlArgs(), node.Metadata.Hostname), exec.Sudo(h))
 }
 
 // CordonNode marks the node unschedulable
@@ -442,6 +483,33 @@ func (h *Host) DeleteNode(node *Host) error {
 	return h.Exec(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "delete node %s", node.Metadata.Hostname), exec.Sudo(h))
 }
 
+// Taints returns all taints added to the node.
+func (h *Host) Taints(node *Host) ([]string, error) {
+	output, err := h.ExecOutput(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), `get node %s -o jsonpath='{range .spec.taints[*]}{.key}={.value}:{.effect}{"\n"}{end}'`, node.Metadata.Hostname), exec.Sudo(h))
+	if err != nil {
+		return nil, fmt.Errorf("failed to get node taints: %w", err)
+	}
+	return strings.Split(strings.TrimSpace(output), "\n"), nil
+}
+
+// AddTaint adds a taint to the node.
+func (h *Host) AddTaint(node *Host, taint string) error {
+	return h.Exec(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "taint nodes --overwrite %s %s", node.Metadata.Hostname, shellescape.Quote(taint)), exec.Sudo(h))
+}
+
+// RemoveTaint removes a taint from the node.
+func (h *Host) RemoveTaint(node *Host, taint string) error {
+	tainted, err := h.Taints(node)
+	if err != nil {
+		return err
+	}
+	if !slices.Contains(tainted, taint) {
+		// Removing a taint not on the node results in an error, so no action is taken
+		return nil
+	}
+	return h.Exec(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "taint nodes %s %s-", node.Metadata.Hostname, shellescape.Quote(taint)), exec.Sudo(h))
+}
+
 // CheckHTTPStatus will perform a web request to the url and return an error if the http status is not the expected
 func (h *Host) CheckHTTPStatus(url string, expected ...int) error {
 	status, err := h.Configurer.HTTPStatus(h, url)
@@ -615,3 +683,29 @@ func (h *Host) FlagsChanged() bool {
 	log.Debugf("%s: installFlags seem to have changed. existing: %+v new: %+v", h, their.Map(), our.Map())
 	return true
 }
+
+// HasHooks returns true when the host has hooks defined for the action and stage.
+func (h *Host) HasHooks(action, stage string) bool {
+    return len(h.Hooks.ForActionAndStage(action, stage)) > 0
+}
+
+// RunHooks runs the hooks for the given action and stage (such as "apply", "before" would run the "before apply" hooks).
+// It respects context cancellation between hook executions.
+func (h *Host) RunHooks(ctx context.Context, action, stage string) error {
+    commands := h.Hooks.ForActionAndStage(action, stage)
+    if len(commands) == 0 {
+        return nil
+    }
+    for _, cmd := range commands {
+        // Abort early if the context has been canceled.
+        if err := ctx.Err(); err != nil {
+            return err
+        }
+
+        log.Infof("%s: running %s %s hook: %q", h, stage, action, cmd)
+        if err := h.Exec(cmd); err != nil {
+            return fmt.Errorf("failed to execute hook %q for action %q stage %q on host %s: %w", cmd, action, stage, h.Address(), err)
+        }
+    }
+    return nil
+}
