@@ -1,8 +1,9 @@
 #!/usr/bin/env sh
 
 K0SCTL_CONFIG=${K0SCTL_CONFIG:-"k0sctl.yaml"}
+OUT=${OUT:-""}
 
-set -e
+set -ex
 
 . ./smoke.common.sh
 trap runCleanup EXIT
@@ -10,7 +11,7 @@ trap runCleanup EXIT
 # custom exit trap to cleanup the backup archives
 runCleanup() {
     cleanup
-    rm k0s_backup*.tar.gz
+    rm -f k0s_backup*.tar.gz || true
 }
 
 deleteCluster
@@ -22,13 +23,31 @@ createCluster
 system_ns_uid=$(bootloose ssh root@manager0 -- k0s kubectl --kubeconfig "/var/lib/k0s/pki/admin.conf" get -n kube-system namespace kube-system -o template='{{.metadata.uid}}')
 node_uid=$(bootloose ssh root@manager0 -- k0s kubectl --kubeconfig "/var/lib/k0s/pki/admin.conf" get node worker0 -o template='{{.metadata.uid}}')
 
-../k0sctl backup --config "${K0SCTL_CONFIG}" --debug
+if [ -z "${OUT}" ]; then
+    echo "Backup with default output filename"
+    ../k0sctl backup --config "${K0SCTL_CONFIG}" --debug
+    RESTORE_FROM="$(ls -t k0s_backup_*.tar.gz 2>/dev/null | head -n1)"
+    if [ ! -f "${RESTORE_FROM}" ]; then
+        echo "Backup archive not found!"
+        exit 1
+    fi
+else
+    RESTORE_FROM="${OUT}"
+  ../k0sctl backup --config "${K0SCTL_CONFIG}" --debug --output "${OUT}"
+fi
+
+echo "Restore from ${RESTORE_FROM} header hexdump:"
+hexdump -C -n 1024 "${RESTORE_FROM}"
 
 # Reset the controller
 bootloose ssh root@manager0 -- k0s stop
 bootloose ssh root@manager0 -- k0s reset
 
-../k0sctl apply --config "${K0SCTL_CONFIG}" --debug --restore-from "$(ls k0s_backup*.tar.gz)"
+echo "Restoring from ${RESTORE_FROM}"
+
+../k0sctl apply --config "${K0SCTL_CONFIG}" --debug --restore-from "${RESTORE_FROM}"
+
+rm -f -- "${RESTORE_FROM}" || true
 
 # Verify kube object UIDs match so we know we did full restore of the API objects
 new_system_ns_uid=$(bootloose ssh root@manager0 -- k0s kubectl --kubeconfig "/var/lib/k0s/pki/admin.conf" get -n kube-system namespace kube-system -o template='{{.metadata.uid}}')
