@@ -15,14 +15,23 @@ echo "* Starting apply"
 ../k0sctl apply --config "${K0SCTL_CONFIG}" --kubeconfig-out applykubeconfig --debug
 echo "* Apply OK"
 
+K0S="k0s"
+
+if [ "${INSTALL_PATH}" != "" ]; then
+    echo "* Checking k0s binary is at the custom install path ${INSTALL_PATH}"
+    bootloose ssh root@manager0 -- ls -al "$(dirname "${INSTALL_PATH}")"
+    bootloose ssh root@manager0 -- test -x "${INSTALL_PATH}"
+    K0S="${INSTALL_PATH}"
+fi
+
 echo "* Verify hooks were executed on the host"
 bootloose ssh root@manager0 -- grep -q hello apply.hook
 
 echo "* Verify 'k0sctl kubeconfig' output includes 'data' block"
-../k0sctl kubeconfig --config k0sctl.yaml | grep -v -- "-data"
+(../k0sctl kubeconfig --debug --trace --config "${K0SCTL_CONFIG}" 2> kubeconfig.log | grep -v -- "-data") || (echo "No data block found in kubeconfig output"; cat "kubeconfig.log"; exit 1)
 
 echo "* Run kubectl on controller"
-bootloose ssh root@manager0 -- k0s kubectl get nodes
+bootloose ssh root@manager0 -- "${K0S}" kubectl get nodes
 
 echo "* Downloading kubectl for local test"
 downloadKubectl
@@ -31,7 +40,7 @@ echo "* Using the kubectl from apply"
 ./kubectl --kubeconfig applykubeconfig get nodes
 
 echo "* Using k0sctl kubecofig locally"
-../k0sctl kubeconfig --config k0sctl.yaml --user smoke --cluster test > kubeconfig
+../k0sctl kubeconfig --config "${K0SCTL_CONFIG}" --user smoke --cluster test > kubeconfig
 
 echo "* Output:"
 grep -v -- -data kubeconfig
