@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env sh
 
 K0SCTL_CONFIG=${K0SCTL_CONFIG:-"k0sctl-dynamic.yaml"}
 
@@ -20,10 +20,10 @@ counter=0
 echo "* Verifying dynamic config reconciliation was a success"
 until ../k0sctl config status -o json --config "${K0SCTL_CONFIG}" | grep -q "SuccessfulReconcile"
 do
-   [[ counter -eq $max_retry ]] && echo "Failed!" && exit 1
+   [ $counter -eq $max_retry ] && echo "Failed!" && exit 1
    echo "* Waiting for a couple of seconds to retry"
    sleep 5
-   ((counter++))
+   counter=$((counter+1))
 done
 
 echo "* OK"
