@@ -1,6 +1,6 @@
 BOOTLOOSE_TEMPLATE=${BOOTLOOSE_TEMPLATE:-"bootloose.yaml.tpl"}
 
-export LINUX_IMAGE="${LINUX_IMAGE:-"quay.io/k0sproject/bootloose-ubuntu20.04"}"
+export LINUX_IMAGE="${LINUX_IMAGE:-"quay.io/k0sproject/bootloose-ubuntu22.04"}"
 export PRESERVE_CLUSTER="${PRESERVE_CLUSTER:-""}"
 export K0S_VERSION
 
@@ -30,6 +30,6 @@ downloadKubectl() {
     case $(uname -m) in
         arm,arm64) ARCH="arm64" ;;
     esac
-    [ -f kubectl ] || (curl -L https://storage.googleapis.com/kubernetes-release/release/v1.28.2/bin/"${OS}"/${ARCH}/kubectl > ./kubectl && chmod +x ./kubectl)
+    [ -f kubectl ] || (curl -L https://dl.k8s.io/release/v1.28.2/bin/${OS}/${ARCH}/kubectl > ./kubectl && chmod +x ./kubectl)
     ./kubectl version --client
 }
